@@ -15,11 +15,10 @@ module Database.Persist.Class.PersistField
     ) where
 
 import Database.Persist.Types.Base
-import Data.Time (Day(..), TimeOfDay, UTCTime)
+import Data.Time (Day(..), TimeOfDay, UTCTime, parseTime)
 #ifdef HIGH_PRECISION_DATE
 import Data.Time.Clock.POSIX (posixSecondsToUTCTime)
 #endif
-import Data.Time.LocalTime (ZonedTime)
 import Data.ByteString.Char8 (ByteString, unpack, readInt)
 import Control.Applicative
 import Data.Int (Int8, Int16, Int32, Int64)
@@ -45,8 +44,16 @@ import qualified Data.Aeson as A
 
 import qualified Data.Set as S
 import qualified Data.Map as M
+import qualified Data.IntMap as IM
 
 import qualified Data.Text.Encoding as TE
+import qualified Data.Vector as V
+
+#if MIN_VERSION_time(1,5,0)
+import Data.Time (defaultTimeLocale)
+#else
+import System.Locale (defaultTimeLocale)
+#endif
 
 -- | A value which can be marshalled to and from a 'PersistValue'.
 class PersistField a where
@@ -65,7 +72,6 @@ instance PersistField String where
     fromPersistValue (PersistDay d) = Right $ Prelude.show d
     fromPersistValue (PersistTimeOfDay d) = Right $ Prelude.show d
     fromPersistValue (PersistUTCTime d) = Right $ Prelude.show d
-    fromPersistValue (PersistZonedTime (ZT z)) = Right $ Prelude.show z
     fromPersistValue PersistNull = Left $ T.pack "Unexpected null"
     fromPersistValue (PersistBool b) = Right $ Prelude.show b
     fromPersistValue (PersistList _) = Left $ T.pack "Cannot convert PersistList to String"
@@ -81,7 +87,7 @@ instance PersistField ByteString where
 
 instance PersistField T.Text where
     toPersistValue = PersistText
-    fromPersistValue = either (Left . T.pack) Right . fromPersistValueText
+    fromPersistValue = fromPersistValueText
 
 instance PersistField TL.Text where
     toPersistValue = toPersistValue . TL.toStrict
@@ -101,24 +107,36 @@ instance PersistField Int8 where
     toPersistValue = PersistInt64 . fromIntegral
     fromPersistValue (PersistInt64 i)  = Right $ fromIntegral i
     fromPersistValue (PersistDouble i) = Right (truncate i :: Int8) -- oracle
+    fromPersistValue (PersistByteString bs) = case readInt bs of  -- oracle
+                                               Just (i,"") -> Right $ fromIntegral i
+                                               xs -> error $ "PersistField Int8 failed parsing PersistByteString xs["++show xs++"] i["++show bs++"]"
     fromPersistValue x = Left $ T.pack $ "int8 Expected Integer, received: " ++ show x
 
 instance PersistField Int16 where
     toPersistValue = PersistInt64 . fromIntegral
     fromPersistValue (PersistInt64 i)  = Right $ fromIntegral i
     fromPersistValue (PersistDouble i) = Right (truncate i :: Int16) -- oracle
+    fromPersistValue (PersistByteString bs) = case readInt bs of  -- oracle
+                                               Just (i,"") -> Right $ fromIntegral i
+                                               xs -> error $ "PersistField Int16 failed parsing PersistByteString xs["++show xs++"] i["++show bs++"]"
     fromPersistValue x = Left $ T.pack $ "int16 Expected Integer, received: " ++ show x
 
 instance PersistField Int32 where
     toPersistValue = PersistInt64 . fromIntegral
     fromPersistValue (PersistInt64 i)  = Right $ fromIntegral i
     fromPersistValue (PersistDouble i) = Right (truncate i :: Int32) -- oracle
+    fromPersistValue (PersistByteString bs) = case readInt bs of  -- oracle
+                                               Just (i,"") -> Right $ fromIntegral i
+                                               xs -> error $ "PersistField Int32 failed parsing PersistByteString xs["++show xs++"] i["++show bs++"]"
     fromPersistValue x = Left $ T.pack $ "int32 Expected Integer, received: " ++ show x
 
 instance PersistField Int64 where
     toPersistValue = PersistInt64 . fromIntegral
     fromPersistValue (PersistInt64 i)  = Right $ fromIntegral i
     fromPersistValue (PersistDouble i) = Right (truncate i :: Int64) -- oracle
+    fromPersistValue (PersistByteString bs) = case readInt bs of  -- oracle
+                                               Just (i,"") -> Right $ fromIntegral i
+                                               xs -> error $ "PersistField Int64 failed parsing PersistByteString xs["++show xs++"] i["++show bs++"]"
     fromPersistValue x = Left $ T.pack $ "int64 Expected Integer, received: " ++ show x
 
 instance PersistField Word where
@@ -150,6 +168,7 @@ instance PersistField Double where
     toPersistValue = PersistDouble
     fromPersistValue (PersistDouble d) = Right d
     fromPersistValue (PersistRational r) = Right $ fromRational r
+    fromPersistValue (PersistInt64 i) = Right $ fromIntegral i
     fromPersistValue x = Left $ T.pack $ "Expected Double, received: " ++ show x
 
 instance (HasResolution a) => PersistField (Fixed a) where
@@ -222,7 +241,12 @@ instance PersistField UTCTime where
     fromPersistValue x@(PersistText t)  =
         case reads $ T.unpack t of
             (d, _):_ -> Right d
-            _ -> Left $ T.pack $ "Expected UTCTime, received " ++ show x
+            _ ->
+                case parse8601 $ T.unpack t of
+                    Nothing -> Left $ T.pack $ "Expected UTCTime, received " ++ show x
+                    Just x -> Right x
+      where
+        parse8601 = parseTime defaultTimeLocale "%FT%T%Q"
     fromPersistValue x@(PersistByteString s) =
         case reads $ unpack s of
             (d, _):_ -> Right d
@@ -230,19 +254,6 @@ instance PersistField UTCTime where
 
     fromPersistValue x = Left $ T.pack $ "Expected UTCTime, received: " ++ show x
 
-instance PersistField ZonedTime where
-    toPersistValue = PersistZonedTime . ZT
-    fromPersistValue (PersistZonedTime (ZT z)) = Right z
-    fromPersistValue x@(PersistText t) =
-        case reads $ T.unpack t of
-            (z, _):_ -> Right z
-            _ -> Left $ T.pack $ "Expected ZonedTime, received " ++ show x
-    fromPersistValue x@(PersistByteString s) =
-        case reads $ unpack s of
-            (z, _):_ -> Right z
-            _ -> Left $ T.pack $ "Expected ZonedTime, received " ++ show x
-    fromPersistValue x = Left $ T.pack $ "Expected ZonedTime, received: " ++ show x
-
 instance PersistField a => PersistField (Maybe a) where
     toPersistValue Nothing = PersistNull
     toPersistValue (Just a) = toPersistValue a
@@ -260,6 +271,11 @@ instance PersistField a => PersistField [a] where
     fromPersistValue (PersistNull) = Right []
     fromPersistValue x = Left $ T.pack $ "Expected PersistList, received: " ++ show x
 
+instance PersistField a => PersistField (V.Vector a) where
+  toPersistValue = toPersistValue . V.toList
+  fromPersistValue = either (\e -> Left ("Vector: " `T.append` e))
+                            (Right . V.fromList) . fromPersistValue
+
 instance (Ord a, PersistField a) => PersistField (S.Set a) where
     toPersistValue = PersistList . map toPersistValue . S.toList
     fromPersistValue (PersistList list) =
@@ -268,6 +284,7 @@ instance (Ord a, PersistField a) => PersistField (S.Set a) where
     fromPersistValue (PersistByteString bs)
         | Just values <- A.decode' (L.fromChunks [bs]) =
             either Left (Right . S.fromList) $ fromPersistList values
+    fromPersistValue PersistNull = Right S.empty
     fromPersistValue x = Left $ T.pack $ "Expected PersistSet, received: " ++ show x
 
 instance (PersistField a, PersistField b) => PersistField (a,b) where
@@ -278,6 +295,10 @@ instance (PersistField a, PersistField b) => PersistField (a,b) where
             Left e          -> Left e
             _               -> Left $ T.pack $ "Expected 2 item PersistList, received: " ++ show v
 
+instance PersistField v => PersistField (IM.IntMap v) where
+    toPersistValue = toPersistValue . IM.toList
+    fromPersistValue = (fmap IM.fromList) . fromPersistValue
+
 instance PersistField v => PersistField (M.Map T.Text v) where
     toPersistValue = PersistMap . map (\(k,v) -> (k, toPersistValue v)) . M.toList
     fromPersistValue = fromPersistMap <=< getPersistMap
@@ -286,8 +307,6 @@ instance PersistField PersistValue where
     toPersistValue = id
     fromPersistValue = Right
 
-deriving instance PersistField (KeyBackend backend entity)
-
 fromPersistList :: PersistField a => [PersistValue] -> Either T.Text [a]
 fromPersistList = mapM fromPersistValue
 
@@ -309,6 +328,7 @@ getPersistMap (PersistMap kvs) = Right kvs
 getPersistMap (PersistText t)  = getPersistMap (PersistByteString $ TE.encodeUtf8 t)
 getPersistMap (PersistByteString bs)
     | Just pairs <- A.decode' (L.fromChunks [bs]) = Right pairs
+getPersistMap PersistNull = Right []
 getPersistMap x = Left $ T.pack $ "Expected PersistMap, received: " ++ show x
 
 data SomePersistField = forall a. PersistField a => SomePersistField a
@@ -321,6 +341,11 @@ instance PersistField Checkmark where
     toPersistValue Inactive = PersistNull
     fromPersistValue PersistNull         = Right Inactive
     fromPersistValue (PersistBool True)  = Right Active
+    fromPersistValue (PersistInt64 1)    = Right Active
+    fromPersistValue (PersistByteString i) = case readInt i of 
+                                               Just (0,"") -> Left $ T.pack "PersistField Checkmark: found unexpected 0 value"
+                                               Just (1,"") -> Right Active
+                                               xs -> Left $ T.pack $ "PersistField Checkmark failed parsing PersistByteString xs["++show xs++"] i["++show i++"]"
     fromPersistValue (PersistBool False) =
       Left $ T.pack "PersistField Checkmark: found unexpected FALSE value"
     fromPersistValue other =
