@@ -10,18 +10,13 @@ import Control.Monad.Trans.Reader hiding (local)
 import Control.Monad.Trans.Resource
 import Control.Monad.Logger
 import Control.Monad.Base
-import Control.Exception.Lifted (onException)
+import Control.Exception.Lifted (onException, bracket)
 import Control.Monad.IO.Class
-import Control.Exception.Lifted (bracket)
 import Control.Exception (mask)
 import System.Timeout (timeout)
-import Control.Monad.Trans.Control (control)
 import Data.IORef (readIORef, writeIORef, newIORef)
 import qualified Data.Map as Map
-import Control.Exception.Lifted (throwIO)
-import Control.Exception (mask)
 import Control.Monad (liftM)
-import System.Timeout (timeout)
 
 -- | Get a connection from the pool, run the given action, and then return the
 -- connection to the pool.
@@ -70,14 +65,16 @@ runSqlPersistM x conn = runResourceT $ runNoLoggingT $ runSqlConn x conn
 runSqlPersistMPool :: SqlPersistM a -> Pool SqlBackend -> IO a
 runSqlPersistMPool x pool = runResourceT $ runNoLoggingT $ runSqlPool x pool
 
+liftSqlPersistMPool :: MonadIO m => SqlPersistM a -> Pool SqlBackend -> m a
+liftSqlPersistMPool x pool = liftIO (runSqlPersistMPool x pool)
+
 withSqlPool :: (MonadIO m, MonadLogger m, MonadBaseControl IO m)
             => (LogFunc -> IO SqlBackend) -- ^ create a new connection
             -> Int -- ^ connection count
             -> (Pool SqlBackend -> m a)
             -> m a
 withSqlPool mkConn connCount f = do
-    pool <- createSqlPool mkConn connCount
-    f pool
+    bracket (createSqlPool mkConn connCount) (liftIO . destroyAllResources) f
 
 createSqlPool :: (MonadIO m, MonadLogger m, MonadBaseControl IO m)
               => (LogFunc -> IO SqlBackend)
